import time
import pytest
import requests
from pygindex.client import IGClient, IGAPIConfig, IGUserAuth, \
    IGResponse, IGSession


@pytest.fixture(scope="module")
def ig_client():
    client = IGClient(IGUserAuth("api_key", "username", "password"),
                      IGAPIConfig("demo"))
    return client


def test_client(ig_client):
    """Check if we have internal data structures set"""
    assert isinstance(ig_client._auth, IGUserAuth)
    assert isinstance(ig_client._api, IGAPIConfig)


def test_request(monkeypatch):
    """Test if request abstraction helper returns correct data structures"""

    class MockResp:
        def __init__(self):
            self.status_code = 200
            self.url = "http://example.com"
            self.headers = {"h_key": "h_value"}

        def json(self):
            return {"d_key": "d_value"}

    def req_get(*args, **kwargs):
        return MockResp()

    monkeypatch.setattr(requests, "get", req_get)

    res = IGClient._request("https://example.com",
                            "get",
                            {"d_key": "d_val"},
                            {"h_key": "h_val"})

    assert isinstance(res, IGResponse)
    assert res.data == {"d_key": "d_value"}
    assert res.headers == {"h_key": "h_value"}


def test_session_invalid_timed_out(ig_client):
    """Test that we detect expired sessions"""
    ig_client._session.expires = int(time.time()) - 1000
    assert ig_client._authentication_is_valid is False


def test_session_invalid_no_token(ig_client):
    """Test that we detect missing security token"""
    session_data = IGSession(cst="abcdefg",
                             expires=int(time.time())+1000)
    ig_client._session = session_data
    assert ig_client._authentication_is_valid is False


def test_session_invalid_no_cst(ig_client):
    """Test that we detect missing CST"""
    session_data = IGSession(security_token="abcdefg",
                             expires=int(time.time())+1000)
    ig_client._session = session_data
    assert ig_client._authentication_is_valid is False


def test_session_valid(ig_client):
    """Test we recognise valid session state"""
    session_data = IGSession(cst="abcdefg",
                             security_token="abcdefg",
                             expires=int(time.time())+1000)
    ig_client._session = session_data
    assert ig_client._authentication_is_valid is True


def test_authentication(monkeypatch, ig_client):
    """Test we can retrieve authentication details"""

    class MockResp:
        def __init__(self):
            self.status_code = 200
            self.url = "http://example.com"
            self.headers = {"Access-Control-Max-Age": "3600",
                            "CST": "abcdefg",
                            "X-SECURITY-TOKEN": "gfedcba"}

        def json(self):
            return {"d_key": "d_value"}

    def req_post(*args, **kwargs):
        return MockResp()

    monkeypatch.setattr(requests, "post", req_post)
    ig_client._authenticate()
    assert ig_client._session.cst == "abcdefg"


def test_authenticated_request(mocker, ig_client):
    """Test we can perform authenticated request"""
    mocker.patch.object(ig_client, "_request", return_value=None)
    res = ig_client._authenticated_request("https://example.com",
                                           "get",
                                           headers={},
                                           data={})
    ig_client._request.assert_called()



import pytest
from pygindex.client import IGAPIConfig


def test_valid_env_var(monkeypatch):
    monkeypatch.setenv("IG_PLATFORM", "demo")
    cfg = IGAPIConfig()
    assert cfg.base_url == "https://demo-api.ig.com/gateway/deal"


def test_invalid_env_var(monkeypatch):
    monkeypatch.setenv("IG_PLATFORM", "does_not_exist")
    with pytest.raises(ValueError):
        cfg = IGAPIConfig()


def test_session_url():
    cfg = IGAPIConfig()
    assert cfg.session_url.endswith("/session")


def test_accounts_url():
    cfg = IGAPIConfig()
    assert cfg.accounts_url.endswith("/accounts")



"""Client module"""

import os
import time
from dataclasses import dataclass, field, fields
from typing import Dict
import requests


@dataclass
class IGUserAuth:
    """User Authentication data"""

    api_key: str = None
    username: str = None
    password: str = None

    def __post_init__(self):
        """Attempt to fill in configuration from env vars"""
        for fld in fields(self):
            if getattr(self, fld.name) is None:
                env_var_name = "IG_{}".format(fld.name.upper())
                if env_var_name not in os.environ:
                    raise ValueError(
                        f"Required argument '{fld.name}' or environment "
                        f"variable '{env_var_name}' not set"
                    )
                setattr(self, fld.name, os.environ[env_var_name])

    @property
    def auth_req_headers(self) -> Dict[str, str]:
        """Construct authentication headers"""
        headers = {
            "X-IG-API-KEY": self.api_key,
            "Content-Type": "application/json",
        }
        return headers

    @property
    def auth_req_data(self) -> Dict[str, str]:
        """Authentication data used in requests"""
        data = {
            "identifier": self.username,
            "password": self.password,
            "encryptedPassword": None,
        }
        return data


@dataclass
class IGAPIConfig:
    """API configuration data"""

    platform: str = None
    base_url: str = field(init=False)

    def __post_init__(self):
        """Attempt to fill in configuration from env vars"""
        platform_urls = {
            "live": "https://api.ig.com/gateway/deal",
            "demo": "https://demo-api.ig.com/gateway/deal",
        }
        if self.platform is None:
            self.platform = os.environ.get("IG_PLATFORM", default="live")
        if self.platform not in platform_urls.keys():
            raise ValueError(
                f"Unknown platform type: {self.platform} "
                f"(valid options: 'live', 'demo')"
            )
        self.base_url = platform_urls[self.platform]

    @property
    def session_url(self) -> str:
        """Session URL"""
        return f"{self.base_url}/session"

    @property
    def accounts_url(self) -> str:
        """Accounts URL"""
        return f"{self.base_url}/accounts"


@dataclass
class IGSession:
    """Session data"""

    cst: str = None
    security_token: str = None
    expires: int = 0


@dataclass
class IGResponse:
    """Response data"""

    data: Dict
    headers: Dict


class IGClient:
    """Client class"""

    def __init__(
        self, auth: IGUserAuth = None, api_config: IGAPIConfig = None
    ):
        self._auth = auth or IGUserAuth()
        self._api = api_config or IGAPIConfig()
        self._session = IGSession()

    @staticmethod
    def _request(
        url: str, method: str, headers: Dict, data: Dict
    ) -> IGResponse:
        """Make a request"""
        req = getattr(requests, method.lower())(
            url, headers=headers, json=data
        )
        response = IGResponse(data=req.json(), headers=req.headers)
        return response

    @property
    def _authentication_is_valid(self) -> bool:
        """Check is we're authenticated, and the authentication is current"""
        current_time = int(time.time())
        if self._session.expires < current_time:
            return False
        if not (self._session.cst and self._session.security_token):
            return False
        return True

    def _authenticated_request(
        self, url: str, method: str, headers: Dict = None, data: Dict = None
    ) -> IGResponse:
        """Authenticate with API if we don't have valid token"""
        if not self._authentication_is_valid:
            self._authenticate()
        if headers:
            hdrs = headers.copy()
        else:
            hdrs = {}
        hdrs.update(
            {
                "CST": self._session.cst,
                "X-SECURITY-TOKEN": self._session.security_token,
            }
        )
        hdrs.update(self._auth.auth_req_headers)
        req = self._request(url=url, method=method, headers=hdrs, data=data)
        return req

    def _authenticate(self):
        """
        Call IG API session URL to retrieve session authentication tokens
        """
        headers = self._auth.auth_req_headers
        data = self._auth.auth_req_data
        req = self._request(
            self._api.session_url, "post", headers=headers, data=data
        )
        if "Access-Control-Max-Age" in req.headers:
            self._session.expires = int(
                time.time() + int(req.headers["Access-Control-Max-Age"])
            )
        self._session.cst = req.headers["CST"]
        self._session.security_token = req.headers["X-SECURITY-TOKEN"]

    def get_session_details(self):
        """Retrieve session details from the API"""
        req = self._authenticated_request(
            url=self._api.session_url, method="get"
        )
        return req.data

    def get_accounts(self):
        """Retrieve account details from the API"""
        req = self._authenticated_request(
            url=self._api.accounts_url, method="get"
        )
        return req.data



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['pygindex']

package_data = \
{'': ['*']}

package_dir = \
{'': 'src'}

setup(name='pygindex',
      version='0.0.1',
      description='PyGIndex - Python library/framework',
      author='Rytis Sileika',
      author_email='rts.pub@pulegium.eu',
      url='https://github.com/rytis/pygindex',
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
     )




import random
import string
import pytest
from pygindex.client import IGUserAuth


def random_str(length=10):
    return "".join(random.choice(string.ascii_letters) for i in range(length))


TEST_INIT_DATA = {"api_key": random_str(),
                  "username": random_str(),
                  "password": random_str()}


@pytest.fixture(scope="module")
def ig_user_auth():
    auth = IGUserAuth(**TEST_INIT_DATA)
    return auth


def test_auth_logon_details(ig_user_auth):
    assert ig_user_auth.username == TEST_INIT_DATA["username"]
    assert ig_user_auth.password == TEST_INIT_DATA["password"]
    assert ig_user_auth.api_key == TEST_INIT_DATA["api_key"]


def test_auth_req_headers(ig_user_auth):
    assert "X-IG-API-KEY" in ig_user_auth.auth_req_headers
    assert "Content-Type" in ig_user_auth.auth_req_headers
    assert ig_user_auth.auth_req_headers["X-IG-API-KEY"] == ig_user_auth.api_key
    assert ig_user_auth.auth_req_headers["Content-Type"] == "application/json"


def test_auth_req_data(ig_user_auth):
    assert "identifier" in ig_user_auth.auth_req_data
    assert "password" in ig_user_auth.auth_req_data
    assert "encryptedPassword" in ig_user_auth.auth_req_data
    assert ig_user_auth.auth_req_data["identifier"] == \
           TEST_INIT_DATA["username"]
    assert ig_user_auth.auth_req_data["password"] == TEST_INIT_DATA["password"]
    assert ig_user_auth.auth_req_data["encryptedPassword"] is None


def test_env_var_lookup(monkeypatch):
    for k, v in TEST_INIT_DATA.items():
        monkeypatch.setenv("IG_{}".format(k.upper()), v)
    auth = IGUserAuth()
    assert auth.username == TEST_INIT_DATA["username"]


def test_undefined_param_exception():
    with pytest.raises(ValueError):
        auth = IGUserAuth()
